import sys

class validation:
    def validate_biosample_id (biosample_id):
        for i in range(len(biosample_id)):
            if 'SAM' not in biosample_id[i]:
                print(f"\033[91mValidation ERROR: {biosample_id[i]} is not a valid biosample accession in line number {i+1}. \033[0m")
                sys.exit()

    def validate_antibiotic_name(antibiotic):
        antibiotic_name = open('ena-CLI/packages/antibiotics.txt','r').read()
        for i in range(len(antibiotic)):
            if antibiotic[i] not in antibiotic_name:
                print(f"\033[91mValidation ERROR: {antibiotic[i]} is not a valid antibiotic_name in line number {i+1}. This must be in {antibiotic_name} \033[0m")
                sys.exit()

    def validate_ast_standard(ast_standard):
        accepted_ast_standards = ['CLSI', 'EUCAST', 'CA-SFM', 'BSAC', 'DIN', 'SIR', 'WRG']
        for i in range(len(ast_standard)):
            if ast_standard[i] not in accepted_ast_standards:
                print(f"\033[91mValidation ERROR: {ast_standard[i]} is not a valid ast_standard in line number {i+1}. This must be in {accepted_ast_standards} \033[0m")
                sys.exit()

    def validate_laboratory_typing_method(laboratory_typing_method):
        laboratory_typing_methods = ['BROTH DILUTION', 'MICROBROTH DILUTION', 'AGAR DILUTION', 'DISC-DIFFUSION', 'NEO-SENSITABS', 'ETEST']
        for i in range(len(laboratory_typing_method)):
            if laboratory_typing_method[i] not in laboratory_typing_methods:
                print(f"\033[91mValidation ERROR: {laboratory_typing_method[i]} is not a valid laboratory_typing_method in line number {i+1}. This must be in {laboratory_typing_methods} \033[0m")
                sys.exit()

    def validate_resistance_phenotype(resistance_phenotype):
        accepted_resistance_phenotype = ['intermediate', 'susceptible', 'resistant', 'non-susceptible', 'not-defined']
        for i in range(len(resistance_phenotype)):
            if resistance_phenotype[i] not in accepted_resistance_phenotype:
                print(f"\033[91mValidation ERROR: {resistance_phenotype[i]}  is not a valid resistance phenotype in line number {i+1}. This must be in {accepted_resistance_phenotype}\033[0m")
                sys.exit()




import pandas as pd
import sys , os
from lxml import etree
import hashlib

class XMLGenerator:
    # -----------------------Calculation of the MD5 hash for a given file
    def calculate_md5(filename_full):
        try:
            # Open the file in binary read mode and read its contents
            with open(filename_full, 'rb') as file:
                file_contents = file.read()
            
            # Calculate the MD5 hash of the file's contents
            file_md5 = hashlib.md5(file_contents).hexdigest()
            
            return file_md5
        
        except Exception as e:
            print(f"\033[91mError: Unable to calculate MD5 hash for file {filename_full}\033[0m")
            print("Exception:", e)
            sys.exit()
    
    @staticmethod
    def other_submission(result_directory, args):
        try:
            manifestFile = pd.read_excel(args.manifestFile, sheet_name="Other Analyses", header = 1)
        except Exception as e:
            print(f"\033[91mError: reading file {args.manifestFile}.\033[0m\nException: {e}")
            sys.exit() 
        
        # Create ANALYSIS_SET element
        analysis_set = etree.Element('ANALYSIS_SET')
        head = [i for i in manifestFile]
        for i in range(len(manifestFile[head[0]])):
            if '#' not in manifestFile[head[0]][i]:
                # Create ANALYSIS element
                analysis = etree.SubElement(analysis_set, 'ANALYSIS', alias=manifestFile['alias'][i])
        
                # Add TITLE and DESCRIPTION elements to ANALYSIS
                if not pd.isna(manifestFile['TITLE'][i]):
                    title = etree.SubElement(analysis, 'TITLE')
                    title.text = manifestFile['TITLE'][i]
            
                if not pd.isna(manifestFile['DESCRIPTION'][i]):
                    description = etree.SubElement(analysis, 'DESCRIPTION')
                    description.text = manifestFile['DESCRIPTION'][i]
        
                # Add STUDY_REF, SAMPLE_REF, and RUN_REF elements to ANALYSIS
                study_ref = etree.SubElement(analysis, 'STUDY_REF', accession=manifestFile['STUDY_REF'][i])
                sample_ref = etree.SubElement(analysis, 'SAMPLE_REF', accession=manifestFile['SAMPLE_REF'][i])

                
                if not pd.isna(manifestFile['RUN_REF'][i]):
                    run_ref = etree.SubElement(analysis, 'RUN_REF', accession=manifestFile['RUN_REF'][i])
        
                # Add ANALYSIS_TYPE element to ANALYSIS
                analysis_type = etree.SubElement(analysis, 'ANALYSIS_TYPE')
                if args.analysisType == 'GENOME_MAP':
                    GENOME_MAP = etree.SubElement(analysis_type, 'GENOME_MAP')
                    PROGRAM = etree.SubElement(GENOME_MAP, 'PROGRAM')
                    PROGRAM.text = manifestFile['PROGRAM'][i]
                    PLATFORM = etree.SubElement(GENOME_MAP, 'PLATFORM')
                    PLATFORM.text = manifestFile['PLATFORM'][i]
                elif args.analysisType == 'REFERENCE_ALIGNMENT':
                    REFERENCE_ALIGNMENT = etree.SubElement(analysis_type, 'REFERENCE_ALIGNMENT')
                    ASSEMBLY = etree.SubElement(REFERENCE_ALIGNMENT, 'ASSEMBLY')
                    STANDARD = etree.SubElement(ASSEMBLY, 'STANDARD' ,accession = manifestFile['ASSEMBLY'][i])
                    SEQUENCE = etree.SubElement(REFERENCE_ALIGNMENT, 'SEQUENCE' , accession = manifestFile['SEQUENCE'][i])
                else:
                    analysis_type_set = etree.SubElement(analysis_type, args.analysisType)
                
        
                # Add FILES element to ANALYSIS
                files = etree.SubElement(analysis, 'FILES')
        
                # Add FILE element to FILES
                file_path = os.path.join(args.inputDir, manifestFile['FILES'][i] )
                checksum = XMLGenerator.calculate_md5(file_path)
                
        
                file = etree.SubElement(files, 'FILE', filename=manifestFile['FILES'][i], filetype=manifestFile['filetype'][i], checksum_method="MD5", checksum=checksum)
        
                # Create XML tree
                xml_tree = etree.ElementTree(analysis_set)
        
                # Write XML tree to file
        output_path = os.path.join(args.inputDir, f'other_submission.xml')
        xml_tree.write(output_path, pretty_print=True, xml_declaration=True, encoding='UTF-8')
        
        print(f"\033[93mSubmission Set XML file has been saved to: {output_path}\033[0m")

        return output_path
        



from lxml import etree
import os
import hashlib
import sys
import platform
import shutil
import pandas as pd
from datetime import datetime

class XML_generator:
    
    # ------------------Method to build the submission XML file
    def project_submission_xml(result_directory,args):
        # Create the root element
        try:
            manifestFile = pd.read_excel(args.manifestFile, sheet_name="project")
        except Exception as e:
            print(f"\033[91mError: reading file {args.manifestFile}.\033[0m\nException: {e}")
            sys.exit() 
            
        submission_set = etree.Element('SUBMISSION_SET')
        submission = etree.SubElement(submission_set, 'SUBMISSION')
        actions = etree.SubElement(submission, 'ACTIONS')
        
        # Create the first ACTION element with ADD
        action1 = etree.SubElement(actions, 'ACTION')
        add1 = etree.SubElement(action1, 'ADD') 
        
        # Create the second ACTION element with HOLD
        head = [i for i in manifestFile]
        for i in range(len(manifestFile[head[0]])):
            if '#' not in manifestFile[head[0]][i]:
                if not pd.isna(manifestFile['date'][i]):
                    action2 = etree.SubElement(actions, 'ACTION')
                    formatted_date = manifestFile['date'][i].strftime('%Y-%m-%d')
                    hold = etree.SubElement(action2, 'HOLD', HoldUntilDate=formatted_date)
    
        # Generate the XML string
        submission_xml = etree.tostring(submission_set, pretty_print=True, xml_declaration=True, encoding='UTF-8')
        
        # Specify the path for the submission XML file
        submission_file = os.path.join(result_directory, 'submission.xml')
    
        # Write to the XML file
        with open(submission_file, 'wb') as f:
            f.write(submission_xml)
    
        print(f"\033[93mSubmission XML file saved to: {submission_file}\033[0m")
        
        # Return the file path
        return submission_file

    #----------------- Method to build project XML file
    def generate_project_xml(result_directory,args):
        try:
            manifestFile = pd.read_excel(args.manifestFile, sheet_name="project")
        except Exception as e:
            print(f"\033[91mError: reading file {args.manifestFile}.\033[0m\nException: {e}")
            sys.exit() 
        # Create the root element
        project_set = etree.Element('PROJECT_SET')

        head = [i for i in manifestFile]
        for i in range(len(manifestFile[head[0]])):
            if '#' not in manifestFile[head[0]][i]:
                # Create the PROJECT element
                project = etree.SubElement(project_set, 'PROJECT', alias=manifestFile['alias'][i])
                
                # Add the NAME element
                if not pd.isna(manifestFile['name'][i]):
                    name = etree.SubElement(project, 'NAME')
                    name.text = manifestFile['name'][i]
                
                # Add the TITLE element
                title = etree.SubElement(project, 'TITLE')
                title.text = manifestFile['Title'][i]
                
                # Add the DESCRIPTION element
                description = etree.SubElement(project, 'DESCRIPTION')
                description.text = manifestFile['description'][i]
                
                # Add the SUBMISSION_PROJECT element
                submission_project = etree.SubElement(project, 'SUBMISSION_PROJECT')
                sequencing_project = etree.SubElement(submission_project, 'SEQUENCING_PROJECT')
                # Add the Locus_Tag element
                if not pd.isna(manifestFile['LocusTag'][i]) and not args.test:
                    # Add the LOCUS_TAG_PREFIX element
                    locus_tag_prefix = etree.SubElement(sequencing_project, 'LOCUS_TAG_PREFIX')
                    locus_tag_prefix.text = manifestFile['LocusTag'][i]
                elif not pd.isna(manifestFile['LocusTag'][i]) and args.test :
                    print(f"\033[91mError: Registration for Locus Tags is not permitted on the test server . \033[0m")
                
                # Add the PROJECT_LINKS element
                if not pd.isna(manifestFile['PubMed'][i]):
                    project_links = etree.SubElement(project, 'PROJECT_LINKS')
                    project_link = etree.SubElement(project_links, 'PROJECT_LINK')
                    xref_link = etree.SubElement(project_link, 'XREF_LINK')
                    db = etree.SubElement(xref_link, 'DB')
                    db.text = "PUBMED"
                    id_element = etree.SubElement(xref_link, 'ID')
                    id_element.text = manifestFile['PubMed'][i]
                
                # Add the PROJECT_ATTRIBUTE element
                if not pd.isna(manifestFile['Study Attributes Tag'][i]) and not pd.isna(manifestFile['Study Attributes Value'][i]):
                    project_attribute = etree.SubElement(project, 'PROJECT_ATTRIBUTE')
                    tag = etree.SubElement(project_attribute, 'TAG')
                    tag.text = manifestFile['Study Attributes Tag'][i]
                    value = etree.SubElement(project_attribute, 'VALUE')
                    value.text = manifestFile['Study Attributes Value'][i]
            
        # Specify the path for the submission XML file
        submission_file = os.path.join(result_directory, 'project.xml')
    
        # Write the XML to the file
        with open(submission_file, 'wb') as f:
            f.write(etree.tostring(project_set, pretty_print=True, xml_declaration=True, encoding='UTF-8'))
    
        print(f"\033[93mProject XML file saved to: {submission_file}\033[0m")
        return submission_file



    # ------------------Method to build the submission XML file
    def build_submission_xml(result_directory):
        # Create XML elements
        submission_set = etree.Element('SUBMISSION')
        actions_elt = etree.SubElement(submission_set, 'ACTIONS')
        action_elt = etree.SubElement(actions_elt, 'ACTION')
        add_elt = etree.SubElement(action_elt, 'ADD')

        # Create XML tree
        submission_xml = etree.ElementTree(submission_set)

        # Specify the path for the submission XML file
        submission_file = os.path.join(result_directory, 'submission.xml')

        # Write to the XML file
        with open(submission_file, 'wb') as f:
            submission_xml.write(f, pretty_print=True, xml_declaration=True, encoding='UTF-8')

        print(f"\033[93mSubmission XML file has been saved to: {submission_file}\033[0m")
        return submission_file

    def generate_sample_xml(result_directory, args):
        try:
            # Read the manifest file into a pandas DataFrame
            manifest = pd.read_excel(args.manifestFile, sheet_name="sample",header=1)
        except Exception as e:
            print(f"\033[91mError: Failed to read the manifest file {args.manifestFile}.\033[0m\nException: {e}")
            sys.exit() 
            
        # Create the root element of the XML document
        sample_set = etree.Element('SAMPLE_SET')
        
        # Iterate over rows in the manifest DataFrame
        for i in range(len(manifest)):
            # Check if the 'tax_id' value is not NaN
            if not pd.isna(manifest['tax_id'][i]):
                # Ignore lines starting with '#'
                if '#' not in str(manifest['tax_id'][i]) :
                    # Create a new 'SAMPLE' element
                    sample = etree.SubElement(sample_set, 'SAMPLE', alias=str(manifest['sample_alias'][i]))
                    
                    # Add 'TITLE' element to the sample
                    title = etree.SubElement(sample, 'TITLE')
                    title.text = str(manifest['sample_title'][i])
                    
                    # Add 'SAMPLE_NAME' element to the sample
                    sample_name = etree.SubElement(sample, 'SAMPLE_NAME')
                    taxon_id = etree.SubElement(sample_name, 'TAXON_ID')
                    taxon_id.text = str(manifest['tax_id'][i])
                    scientific_name = etree.SubElement(sample_name, 'SCIENTIFIC_NAME')
                    scientific_name.text = str(manifest['scientific_name'][i])
                    
                    # Add 'SAMPLE_ATTRIBUTES' element to the sample
                    sample_attributes = etree.SubElement(sample, 'SAMPLE_ATTRIBUTES')
                    sample_attribute = etree.SubElement(sample_attributes, 'SAMPLE_ATTRIBUTE')
                    tag = etree.SubElement(sample_attribute, 'TAG')
                    tag.text = "ENA-CHECKLIST"
                    value = etree.SubElement(sample_attribute, 'VALUE')

                    manifestFile = pd.read_excel(args.manifestFile, sheet_name="sample")
                    manifestFile_head = [h for h in manifestFile]
                    for e in manifestFile_head:
                        if 'ERC' in e:
                            value.text = e
                                
                    # Add 'SAMPLE_ATTRIBUTE' elements to the sample for each attribute in the manifest
                    head = [h for h in manifest]
                    for j in range(4, len(head)):
                        if not pd.isna(manifest[head[j]][i]):
                            sample_attribute = etree.SubElement(sample_attributes, 'SAMPLE_ATTRIBUTE')
                            tag = etree.SubElement(sample_attribute, 'TAG')
                            tag.text = head[j]
                            value = etree.SubElement(sample_attribute, 'VALUE')
                            if head[j] in ['collection date','receipt date'] :
                                value.text = manifest[head[j]][i].strftime('%Y-%m-%d')
                            else: 
                                value.text = str(manifest[head[j]][i])
                            if head[j] in ['geographic location (latitude)','geographic location (longitude)']:
                                units = etree.SubElement(sample_attribute, 'UNITS')
                                units.text = 'DD'
                            elif head[j] == 'host age':
                                units = etree.SubElement(sample_attribute, 'UNITS')
                                units.text = 'years'
                            
        # Create the XML tree
        sample_xml = etree.ElementTree(sample_set)
        
        # Specify the path for the submission XML file
        sample_file = os.path.join(result_directory, 'sample.xml')
        
        # Write the XML tree to the XML file
        with open(sample_file, 'wb') as f:
            sample_xml.write(f, pretty_print=True, xml_declaration=True, encoding='UTF-8')
        
        print(f"\033[93mThe sample XML file has been saved to: {sample_file}\033[0m")
        return sample_file


    # -----------------------Calculation of the MD5 hash for a given file
    def calculate_md5(filename_full):
        try:
            # Open the file in binary read mode and read its contents
            with open(filename_full, 'rb') as file:
                file_contents = file.read()
            
            # Calculate the MD5 hash of the file's contents
            file_md5 = hashlib.md5(file_contents).hexdigest()
            
            return file_md5
        
        except Exception as e:
            print(f"\033[91mError: Unable to calculate MD5 hash for file {filename_full}\033[0m")
            print("Exception:", e)
            sys.exit()
            
    # --------------------Method to generate the set XML file
    def antibiogram_generate_set_xml(result_directory, args):
        # Create ANALYSIS_SET element
        analysis_set = etree.Element('ANALYSIS_SET')

        # Create ANALYSIS element
        analysis = etree.SubElement(analysis_set, 'ANALYSIS', alias=args.alias)

        # Add TITLE and DESCRIPTION elements to ANALYSIS
        title = etree.SubElement(analysis, 'TITLE')
        if args.Title:
            title.text = args.Title
        else:
            title.text = f'Antibiogram for study {args.Study}, sample {args.sample}' 
        if args.description:
            description = etree.SubElement(analysis, 'DESCRIPTION')
            description.text = args.description

        # Add STUDY_REF, SAMPLE_REF, and RUN_REF elements to ANALYSIS
        study_ref = etree.SubElement(analysis, 'STUDY_REF', accession=args.Study)
        sample_ref = etree.SubElement(analysis, 'SAMPLE_REF', accession=args.sample)

        # Add ANALYSIS_TYPE element to ANALYSIS
        analysis_type = etree.SubElement(analysis, 'ANALYSIS_TYPE')
        AMR_ANTIBIOGRAM = etree.SubElement(analysis_type, 'AMR_ANTIBIOGRAM')

        # Add FILES element to ANALYSIS
        files = etree.SubElement(analysis, 'FILES')

        # Add FILE element to FILES
        checksum = XML_generator.calculate_md5(args.filename)
        try:
            shutil.copy(args.filename, result_directory)
            filename = os.path.basename(args.filename)
        except Exception as e: 
            print(f"\033[91mError: Unable to copy the file {args.filename} to {result_directory} \033[0m")
            sys.exit()

        file = etree.SubElement(files, 'FILE', filename=filename, filetype="tab", checksum_method="MD5", checksum=checksum)

        # Create XML tree
        xml_tree = etree.ElementTree(analysis_set)

        # Write XML tree to file
        output_path = os.path.join(result_directory, f'Antibiogram_for_{args.sample}.xml')
        xml_tree.write(output_path, pretty_print=True, xml_declaration=True, encoding='UTF-8')

        print(f"\033[93mSubmission Set XML file has been saved to: {output_path}\033[0m")
        return output_path



import pandas as pd
import os , sys
import subprocess

class webin:

    def webin_submission(manifest, args, webin_cli, sample, type):
        # Build the command based on the provided arguments
        command = [
            f"java -jar {webin_cli}",
            f"-context {type}",
            f"-manifest \"{manifest}\"",
            f"-inputDir {args.inputDir}",
            f"-userName {args.username}",
            f"-password {args.password}"
        ]
    
        # Add optional arguments if provided
        if args.test:
            command.append("-test")
        if args.centerName:
            command.append(f"-centerName '{args.centerName}'")
    
        # Execute the command - validation
        try:
            subprocess.run(" ".join(command+['-validate']), shell=True, check=True)
            print(f"\033[93m\nValidation successful. {sample}\033[0m")
        except subprocess.CalledProcessError as e:
            print(f"\033[91m\nValidation error. {sample}\033[0m\nException: {e}")
            sys.exit()

        # Execute the command - submission
        try:
            subprocess.run(" ".join(command+['-submit']), shell=True, check=True)
            print(f"\033[93m\nSubmission successful. {sample}\033[0m")
        except subprocess.CalledProcessError as e:
            print(f"\033[91m\nSubmission error. {sample}\033[0m\nException: {e}")
            sys.exit()
        
    def run_submission(result_directory,args, webin_cli):
        try:
            manifestFile = pd.read_excel(args.manifestFile, sheet_name="run",header = 1)
        except Exception as e:
            print(f"\033[91mError: reading file {args.manifestFile}.\033[0m\nException: {e}")
            sys.exit() 
        head = [i for i in manifestFile]
        for i in range(len(manifestFile['sample'])):
            if '#' not in manifestFile[head[0]][i]:
                if not pd.isna(manifestFile['sample'][i]):
                    if ' ' in manifestFile['sample'][i]:
                        name = manifestFile['sample'][i].replace(' ','_')
                    else: 
                        name = manifestFile['sample'][i]
                    manifest = os.path.join(result_directory, f"manifest_run_{name}.txt")
                    if '\\' in manifest :
                        manifest =manifest.replace ('\\','/')
                    with open(manifest , 'w') as file:
                        for j in range(len(head)) : 
                            if not pd.isna(manifestFile[head[j]][i]):
                                if 'file_name' in head[j]:
                                    #file_name = os.path.join('../',args.inputDir,manifestFile[head[j]][i])
                                    file_name = manifestFile[head[j]][i]
                                    file.write(f"FASTQ\t{file_name}\n")
                                elif head[j].upper() == 'INSERT_SIZE':
                                    file.write(f'{head[j].upper()}\t{int(manifestFile[head[j]][i])}\n')
                                else:
                                    file.write(f'{head[j].upper()}\t{manifestFile[head[j]][i]}\n')
    
                webin.webin_submission(manifest , args, webin_cli, manifestFile['sample'][i],'reads')

    
    def genome_submission(result_directory,args, webin_cli):
        try:
            manifestFile = pd.read_excel(args.manifestFile, sheet_name="genome")
        except Exception as e:
            print(f"\033[91mError: reading file {args.manifestFile}.\033[0m\nException: {e}")
            sys.exit() 
        head = [i for i in manifestFile]
        for i in range(len(manifestFile['SAMPLE'])):
            if '#' not in manifestFile[head[0]][i]:
                if not pd.isna(manifestFile['SAMPLE'][i]):
                    manifest = os.path.join(result_directory, f"manifest_genome_{manifestFile['SAMPLE'][i]}.txt")
                    if '\\' in manifest :
                        manifest =manifest.replace ('\\','/')
                    with open(manifest , 'w') as file:
                        for j in range(len(head)) : 
                            if not pd.isna(manifestFile[head[j]][i]):
                                if head[j].upper() == 'MINGAPLENGTH':
                                    file.write(f'{head[j].upper()}\t{int(manifestFile[head[j]][i])}\n')
                                else:
                                    file.write(f'{head[j].upper()}\t{manifestFile[head[j]][i]}\n')

                webin.webin_submission(manifest , args, webin_cli, manifestFile['SAMPLE'][i], args.context)

    def targeted_submission(result_directory,args, webin_cli):
        try:
            manifestFile = pd.read_excel(args.manifestFile, sheet_name="targeted")
        except Exception as e:
            print(f"\033[91mError: reading file {args.manifestFile}.\033[0m\nException: {e}")
            sys.exit() 
        head = [i for i in manifestFile]
        for i in range(len(manifestFile['NAME'])):
            if '#' not in manifestFile[head[0]][i]: 
                if not pd.isna(manifestFile['NAME'][i]):
                    manifest = os.path.join(result_directory, f"manifest_targeted_{manifestFile['NAME'][i]}.txt")
                    with open(manifest , 'w') as file:
                        for j in range(len(head)) : 
                            if not pd.isna(manifestFile[head[j]][i]):
                                file.write(f'{head[j].upper()}\t{manifestFile[head[j]][i]}\n')

                webin.webin_submission(manifest , args, webin_cli, manifestFile['NAME'][i], 'sequence')
            
        


import subprocess
import sys

class submission:
    def submit_to_ENA(submission_file , submission_set_file , args, type):
        
        # Construct the curl command
        if args.test:
        
            curl_command = [
                'curl',
                '-u', f'{args.username}:{args.password}',
                '-F', f'SUBMISSION=@{submission_file}',
                '-F', f'{type}=@{submission_set_file}',
                'https://wwwdev.ebi.ac.uk/ena/submit/drop-box/submit/'
            ]
        else:
            curl_command = [
                'curl',
                '-u', f'{args.username}:{args.password}',
                '-F', f'SUBMISSION=@{submission_file}',
                '-F', f'{type}=@{submission_set_file}',
                'https://www.ebi.ac.uk/ena/submit/drop-box/submit/'
            ]

        # Execute the curl command
        try:
            subprocess.run(curl_command, check=True)
            #print("\033[93mSubmission successful.\033[0m")
        except subprocess.CalledProcessError as e:
            print(f"\033[91mError: Submission failed.\033[0m\nException: {e}")
            sys.exit()




import argparse
import os , sys
import pandas as pd

from .packages.XMLGenerator import XML_generator
from .packages.submission import submission
from .packages.antibiogram import validation
from .packages.webin import webin
from .others.XMLGenerator import XMLGenerator

# color https://gist.github.com/Prakasaka/219fe5695beeb4d6311583e79933a009

# run 
# https://github.com/enasequence/webin-cli/releases/latest 
webin_cli = 'ena-CLI/packages/webin-cli-7.0.1.jar'


def get_args(): # https://docs.python.org/3/library/argparse.html
    epilog = ("""
            \033[93mNote:
            (1) \033[0mPlease register your project and sample object before registering the antibiogram. 
            Here is the link for more information: \033[94mhttps://ena-docs.readthedocs.io/en/latest/\033[0m.
            \033[93m(2) \033[0mPlease make sure to provide the title and description using single quotes if they contain spaces or special characters. 
            For any error or assistance please contact the ENA helpdesk: \033[94mhttps://www.ebi.ac.uk/ena/browser/support \033[0m
        """) 
    parser = argparse.ArgumentParser(description='\033[93mThis script facilitates the submission of projects, samples, runs, assemblies, and other analyses to the public repository ENA (European Nucleotide Archive). It also assists in validating AMR (Antimicrobial Resistance) antibiograms before submission.\033[0m'
                                     ,epilog=epilog)
    
    # Create subparsers
    subparsers = parser.add_subparsers(title='\033[93mSubcommands\033[0m', dest='subcommand')
    
    # 1. Project submission
    project = subparsers.add_parser('project', help='Register a Study (Project)',epilog=epilog)
    
    #### Required arguments
    mandatory_p = project.add_argument_group('\033[93mMandatory arguments\033[0m')
    mandatory_p.add_argument('-u', '--username', type=str, help='Webin submission account (e.g., Webin-XXX)', required=True)
    mandatory_p.add_argument('-p', '--password', type=str, help='Password for the submission account', required=True)
    mandatory_p.add_argument('-m', '--manifestFile', type=str, help='Manifest file (template: templates/templates.xlsx)', required=True)

    ##### Optional arguments
    optional_p = project.add_argument_group('\033[93mOptional arguments\033[0m')
    optional_p.add_argument('-t', '--test', action='store_true', help='use Webin test service instead of the production service. Please note that the Webin upload area is shared between test and production services, and that test submission files will not be archived. (Optional)')
    
    
    # 2. Sample submission
    sample = subparsers.add_parser('sample', help='Register samples',epilog=epilog)
    #### Required arguments
    mandatory_s = sample.add_argument_group('\033[93mMandatory arguments\033[0m')
    mandatory_s.add_argument('-u', '--username', type=str, help='Webin submission account (e.g., Webin-XXX)', required=True)
    mandatory_s.add_argument('-p', '--password', type=str, help='Password for the submission account', required=True)
    mandatory_s.add_argument('-m', '--manifestFile', type=str, help='Manifest file (template: templates/templates.xlsx) - Sample manifest file (template on: \033[94mhttps://www.ebi.ac.uk/ena/browser/checklist\033[0m)', required=True)

    #### Optional arguments
    optional_s = sample.add_argument_group('\033[93mOptional arguments\033[0m')
    optional_s.add_argument('-t', '--test', action='store_true', help='use Webin test service instead of the production service. Please note that the Webin upload area is shared between test and production services, and that test submission files will not be archived. (Optional)')

    # 3. run submission
    run = subparsers.add_parser('run', help='Register run',epilog=epilog)
    #### Required arguments
    mandatory_r = run.add_argument_group('\033[93mMandatory arguments\033[0m')
    mandatory_r.add_argument('-u', '--username', type=str, help='Webin submission account (e.g., Webin-XXX)', required=True)
    mandatory_r.add_argument('-p', '--password', type=str, help='Password for the submission account', required=True)
    mandatory_r.add_argument('-m', '--manifestFile', type=str, help='Manifest file (template: templates/templates.xlsx)', required=True)
    mandatory_r.add_argument('-i', '--inputDir', type=str, help=' input directory for files declared in manifest file', required=True)
    

    #### Optional arguments
    optional_r = run.add_argument_group('\033[93mOptional arguments\033[0m')
    optional_r.add_argument('-C', '--centerName', type=str, help=' the center name of the submitter (mandatory for broker accounts).')
    optional_r.add_argument('-t', '--test', action='store_true', help='use Webin test service instead of the production service. Please note that the Webin upload area is shared between test and production services, and that test submission files will not be archived. (Optional)')
    
    # 4. Genome Assemblies 
    Genome = subparsers.add_parser('genome', help='Submitting Genome Assemblies of Individuals or Cultured Isolates - Metagenome Assemblies - Environmental Single-Cell Amplified Genomes - Transcriptome Assemblies - Metatranscriptome Assemblies',epilog=epilog)
    #### Required arguments
    mandatory_g = Genome.add_argument_group('\033[93mMandatory arguments\033[0m')
    mandatory_g.add_argument('-u', '--username', type=str, help='Webin submission account (e.g., Webin-XXX)', required=True)
    mandatory_g.add_argument('-p', '--password', type=str, help='Password for the submission account', required=True)
    mandatory_g.add_argument('-m', '--manifestFile', type=str, help='Manifest file (template: templates/templates.xlsx)', required=True)
    mandatory_g.add_argument('-i', '--inputDir', type=str, help=' input directory for files declared in manifest file', required=True)
    mandatory_g.add_argument('-c', '--context', choices=['genome','transcriptome'], help=' the assembly submission type', required=True)
    

    #### Optional arguments
    optional_g = Genome.add_argument_group('\033[93mOptional arguments\033[0m')
    optional_g.add_argument('-C', '--centerName', type=str, help=' the center name of the submitter (mandatory for broker accounts).')
    optional_g.add_argument('-t', '--test', action='store_true', help='use Webin test service instead of the production service. Please note that the Webin upload area is shared between test and production services, and that test submission files will not be archived. (Optional)')
    
    # 5. Submit Targeted Sequences
    Targeted = subparsers.add_parser('targeted', help='Submit Targeted Sequences',epilog=epilog)
    
    #### Required arguments
    mandatory_t = Targeted.add_argument_group('\033[93mMandatory arguments\033[0m')
    mandatory_t.add_argument('-u', '--username', type=str, help='Webin submission account (e.g., Webin-XXX)', required=True)
    mandatory_t.add_argument('-p', '--password', type=str, help='Password for the submission account', required=True)
    mandatory_t.add_argument('-m', '--manifestFile', type=str, help='Manifest file (template: templates/templates.xlsx)', required=True)
    mandatory_t.add_argument('-i', '--inputDir', type=str, help=' input directory for files declared in manifest file', required=True)
    

    #### Optional arguments
    optional_t = Targeted.add_argument_group('\033[93mOptional arguments\033[0m')
    optional_t.add_argument('-C', '--centerName', type=str, help=' the center name of the submitter (mandatory for broker accounts).')
    optional_t.add_argument('-t', '--test', action='store_true', help='use Webin test service instead of the production service. Please note that the Webin upload area is shared between test and production services, and that test submission files will not be archived. (Optional)')

    # 6. Other Analyses
    # 7. antibiograms submission
    Other = subparsers.add_parser('other', help='Submit Other Analyses',epilog=epilog)
    
    #### Required arguments
    mandatory_o = Other.add_argument_group('\033[93mMandatory arguments\033[0m')
    mandatory_o.add_argument('-u', '--username', type=str, help='Webin submission account (e.g., Webin-XXX)', required=True)
    mandatory_o.add_argument('-p', '--password', type=str, help='Password for the submission account', required=True)
    mandatory_o.add_argument('-m', '--manifestFile', type=str, help='Manifest file (template: templates/templates.xlsx)', required=True)
    mandatory_o.add_argument('-a', '--analysisType', choices=['GENOME_MAP','REFERENCE_ALIGNMENT','SEQUENCE_ANNOTATION','ASSEMBLY_GRAPH','PROCESSED_READ','PATHOGEN_ANALYSIS','AMR_ANTIBIOGRAM','COVID-19_FILTERED_VCF','COVID-19_CONSENSUS','PHYLOGENY_ANALYSIS'], help='The analysis type is provided in the XML itself. It must be one of the following', required=True)
    mandatory_o.add_argument('-i', '--inputDir', type=str, help=' input directory for files declared in manifest file', required=True)


    #### Optional arguments
    optional_o = Other.add_argument_group('\033[93mOptional arguments\033[0m')
    optional_o.add_argument('-t', '--test', action='store_true', help='use Webin test service instead of the production service. Please note that the Webin upload area is shared between test and production services, and that test submission files will not be archived. (Optional)')
    
    #args = parser.parse_args()
    # Add version argument
    parser.add_argument('-v', '--version', action='version', version='%(prog)s 1.0')

    return parser


def process():
    if len(sys.argv) <= 1: 
        print(get_args().print_help())
        sys.exit()
    args = get_args().parse_args()

    result_directory = 'results'  
    if not os.path.exists(result_directory):
        os.makedirs(result_directory)
    result_directory = os.path.abspath(result_directory)

    if args.subcommand == 'project':
        submission_file = XML_generator.project_submission_xml(result_directory,args)
        submission_set_file = XML_generator.generate_project_xml (result_directory, args)
        submission.submit_to_ENA(submission_file , submission_set_file, args, 'PROJECT')

    elif args.subcommand == 'sample':
        submission_file = XML_generator.build_submission_xml(result_directory)
        submission_set_file = XML_generator.generate_sample_xml(result_directory,args)
        submission.submit_to_ENA(submission_file , submission_set_file, args , 'SAMPLE')

    elif args.subcommand == 'run':
        webin.run_submission(result_directory, args  , webin_cli)

    elif args.subcommand == 'genome':
        webin.genome_submission(result_directory, args  , webin_cli)

    elif args.subcommand == 'targeted':
        webin.targeted_submission(result_directory, args  , webin_cli)

    elif args.subcommand == 'other':
        if args.analysisType == 'AMR_ANTIBIOGRAM' :
            try:
                #antibiogram_file = pd.read_csv (args.filename, sep='\t')
                manifestFile = pd.read_excel(args.manifestFile, sheet_name="Other Analyses", header = 1)
            except Exception as e:
                print(f"\033[91mError: reading file {args.manifestFile}.\033[0m\nException: {e}")
                sys.exit() 
            head = [i for i in manifestFile]
            for i in range(len(manifestFile[head[0]])):
                if '#' not in manifestFile[head[0]][i]:
                    file_path = os.path.join(args.inputDir, manifestFile['FILES'][i] )
                    try: 
                        antibiogram_file = pd.read_csv (file_path, sep='\t')
                    except Exception as e:
                        print(f"\033[91mError: reading file {file_path}.\033[0m\nException: {e}")
                        sys.exit()
                    validation.validate_biosample_id(antibiogram_file['bioSample_ID'])
                    validation.validate_antibiotic_name (antibiogram_file['antibiotic_name'])
                    validation.validate_ast_standard(antibiogram_file['ast_standard'])
                    validation.validate_resistance_phenotype(antibiogram_file['resistance_phenotype'])

        
        submission_file = XML_generator.build_submission_xml(args.inputDir)
        submission_set_file= XMLGenerator.other_submission(result_directory, args )
        submission.submit_to_ENA(submission_file , submission_set_file, args , 'ANALYSIS')
        

    print(f"\033[93m\n\n-------[ done ]-------\033[0m")
    print(f"\033[94mFor any error or assistance please contact the ENA helpdesk: https://www.ebi.ac.uk/ena/browser/support \n\n\033[0m")


#if __name__ == '__main__':
#   process()
    


from setuptools import setup, find_packages

with open ('readme.txt','r') as file:
    description = file.read()

setup(
    name='enaCLI',
    version='1.0.0',
    packages=find_packages(),
    install_requires=[
        'argparse',
        'pandas',
        'lxml',
    ],
    entry_points={
        'console_scripts': [
            'enaCLI = enaCLI.__main__:main',
        ],
    },
    author='Khadim GUEYE',
    author_email='gueye.kgy@gmail.com',
    description='This script facilitates the submission of projects, samples, runs, assemblies, and other analyses to the public repository ENA (European Nucleotide Archive). It also assists in validating AMR (Antimicrobial Resistance) antibiograms before submission.',
    url='https://github.com/KhadimGueyeKGY/ena-CLI',
    long_description = description,
    long_description_content_type = 'text/markdown',
)
